#!/bin/ksh
# Why korn shell? Because of 100% portability: /bin/sh could be whatever.
# Get pdksh or some other ksh88 compatible shell if you lack /bin/ksh.
# written by Martin Macok <martin.macok@underground.cz>

# TMPDIR=/path/to/secure_writable_dir
TMPDIR="${TMP:-/tmp}"

# local config can (and should) override this
LOGFILE="${LOGFILE:-stdout}"

CONFDIR="/etc/echelog/modules"
CONFIG="${CONFDIR}/alert.sh.conf"
# LOCAL_CONFIG=./alert.sh.local-conf

####################
# read configuration

. "$CONFIG"
# . "$LOCAL_CONFIG"  # must be explicitely loaded with -C

umask 077

###########
# functions

function printhelp {
echo "Alert Echelog module.
Usage: alert.sh [-h|--help] [-C|--config file] [-l file|stdout] [-b string] 
		[-e] [-email email@address]
		[-s] [-sms \"smssend args\"] [-smsok \"smssend okay regexp\"]
		     [-smsretry number] [-smsretry_int seconds]
		[-ocmd \"other command\"]
		[-w] [-wuser user]
		[-p] [-psound soundfile]
                [-x] [-xdisplay DISPLAY]

alert.sh expects input message on stdin. It performs no action by default.
Actions can be turned on in (local) configuration file or on cmdline.

options:
 -h, --help              ... print this help
 -C, --config file       ... read local configuration file
 -b string               ... set brief message (ie. subject or title)
 -o                      ... run other command
 -e                      ... send email
 -s                      ... send SMS
 -sfe                    ... send email on SMS fail
 -w                      ... write user
 -x                      ... pop up xmessage
 -p                      ... play sound
 -email email@address    ... email address to send alert to
 -sms \"smssend args\"   ... arguments to smssend (provider & phone number)
 -smsok \"okay regexp\"  ... what smssend tells when message sent (regexp)
 -smsretry number        ... how many times to retry sending sms
 -smsretry_int seconds   ... delay between retrying
 -smsemail em@il         ... email where to send if sending sms failes
 -ocmd \"command\"       ... which other command to run
 -wuser \"user\"         ... which user to write (unix write)
 -psound soundfile       ... which sound file to play
 -xdisplay               ... which remote display to use for xmessage"
}


# logging routines
if [ "x$LOGFILE" = "xstdout" ] ; then
  function log    { [ -z "$@" ] || echo "$@" ; }
  function catlog { cat ; }
  function error  { echo "ERROR: $@" >&2 ; }
  function abort  { echo "FATAL ERROR: $@. Aborting." >&2 ; exit 1 ; }
else
  function log    { [ -z "$@" ] || echo "$@" >"$LOGFILE" ; }
  function catlog { cat  >"$LOGFILE" ; }
  function error  { echo "ERROR: $@" >"$LOGFILE" ; }
  function abort  { echo "FATAL ERROR: $@. Aborting." >"$LOGFILE" ; exit 1 ; }
fi

# usage: make_temp [-d] MKTEMP_ARG_FILE
# returns 0 on success, 1 on failure
function make_temp
{
    if [ -x "`which mktemp 2>/dev/null`" ] ; then
        mktemp "$@" && return 0
        return 1
    fi

    if [ ! -z "$TMPDIR" ] && \
       [ "$TMPDIR" != "/tmp" -a "$TMPDIR" != "/var/tmp" ] && \
       [ -d "$TMPDIR" -a -w "$TMPDIR" ]; then
        if [ "x$1" = "x-d" ] ; then
            MAKEIT=mkdir
            shift
        else
            MAKEIT=touch
        fi
        TF="${@}.$$"
        $MAKEIT "$TF" && echo "$TF" && return 0
        echo "ERROR: Can't create temporary file $TF" >&2
        return 1
    else
        echo >&2
        echo "You should install mktemp from ftp://ftp.mktemp.org/pub/mktemp/" >&2
        echo "See Echelog documentation (Requirements)" >&2
        echo >&2
        echo "OR">&2
        echo >&2
        echo "Change \$TMPDIR environment variable to secure writable directory" >&2
        return 1
    fi
}


####################
# parse command line

while [ "$#" -gt 0 ] ; do
        if [ "x$1" = "x-h"  -o  "x$1" = "x--help" ] ; then
                printhelp
                exit
        elif [ "x$1" = "x-C" -o "x$1" = "x--config" ] ; then
                if [ -f "${CONFDIR}/$2" ] ; then
                        . "${CONFDIR}/$2"
		elif [ -f "$2" ] ; then
                        . "$2"
		else
                        error "Configuration file $2 doesn't exist."
                fi
                shift
        elif [ "x$1" = "x-b" ] ; then
                BRIEFMSG="$2"
                shift
        elif [ "x$1" = "x-e" ] ; then
                SEND_EMAIL=yes
        elif [ "x$1" = "x-s" ] ; then
                SEND_SMS=yes
        elif [ "x$1" = "x-sfe" ] ; then
                SMS_EMAIL_ON_FAIL=yes
        elif [ "x$1" = "x-w" ] ; then
                SEND_WRITE=yes
        elif [ "x$1" = "x-x" ] ; then
                SEND_XMESSAGE=yes
        elif [ "x$1" = "x-p" ] ; then
                SEND_SOUND=yes
        elif [ "x$1" = "x-o" ] ; then
                SEND_OTHER_CMD="yes"
        elif [ "x$1" = "x-email" ] ; then
		EMAIL_ADDRESS="$2" # TODO test it?
		shift
        elif [ "x$1" = "x-sms" ] ; then
		SMSSEND_ARGS="$2"
		shift
        elif [ "x$1" = "x-smsok" ] ; then
		SMSSEND_OK="$2"
		shift
        elif [ "x$1" = "x-smsretry" ] ; then
		SMSSEND_RETRY="$2"
		shift
        elif [ "x$1" = "x-smsretry_int" ] ; then
		SMSSEND_RETRY_INT="$2"
		shift
        elif [ "x$1" = "x-smsemail" ] ; then
		SMSSEND_EMAIL="$2"
		shift
        elif [ "x$1" = "x-ocmd" ] ; then
		RUN_OTHER_CMD="$2"
		shift
        elif [ "x$1" = "x-ocmd" ] ; then
		RUN_OTHER_CMD="$2"
		shift
        elif [ "x$1" = "x-wuser" ] ; then
		WRITE_USER="$2"
		shift
        elif [ "x$1" = "x-psound" ] ; then
		SOUNDFILE="$2"
		shift
        elif [ "x$1" = "x-xdisplay" ] ; then
		XMESSAGE_REMOTE_DISPLAY="$2"
		shift
        else
                error "Unknown command line parametr: $1"
                shift
        fi
	shift
done

# prepare for work

FMSG=$(make_temp "$TMPDIR"/alert_msg_XXXXXX)
[ -z "$FMSG" ] && abort "Cannot create message file in $TMP"
trap "rm -f $FMSG ; exit" 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15

cat >"$FMSG"
MSG=$(cat "$FMSG")
if [ -z "$MSG" ] ; then
        error "Empty message"
        log "Using (null) message"
        MSG="(null)"
        echo "$MSG" >>"$FMSG"
fi

#################
# log the message

log "======================================================"
log "Date: `date +\"%Y-%m-%d %H:%M:%S\"`"
log "Alert message (`wc -c <\"$FMSG\"|tr -d " "` bytes):"
log "------------------------------------------------------"
catlog <"$FMSG"
log "------------------------------------------------------"

log "Actions:"

############
# play sound

if [ "$SEND_SOUND" = "yes" ] ; then
        if [ -z "$SOUND_FILE" -a -f "$SOUND_FILE" ] ; then
                error "SOUND_FILE $SOUND_FILE does not exist"
        else
                log "Playing sound ($SOUND_FILE)"
                ( "$SOUND_CMD" "$SOUND_FILE" 2>&1 | catlog ) &
        fi
fi

############
# send email

if [ "$SEND_EMAIL" = "yes" ] ; then
        if [ -z "$EMAIL_ADDRESS" ] ; then
                error "EMAIL_ADDRESS not set, using root"
                EMAIL_ADDRESS=root
        fi
        if [ -z "$EMAIL_CMD" ] ; then
                error "EMAIL_CMD not defined, using 'mail'"
                EMAIL_CMD=mail
        fi
        log "Sending email to $EMAIL_ADDRESS"
        "$EMAIL_CMD" -s "${BRIEFMSG:-alert}" "$EMAIL_ADDRESS" <"$FMSG"|catlog
fi

#######
# write

if [ "$SEND_WRITE" = "yes" ] ; then
        if [ -z "$WRITE_USER" -o -z "$WRITE_CMD" ] ; then
                error "WRITE_USER or WRITE_CMD not set"
        else
                log "Writing message to $WRITE_USER"
                { sleep 1 ; 
                  echo "${BRIEFMSG:-alert}" ; 
                  cat "$FMSG" ; } | "$WRITE_CMD" "$WRITE_USER" |catlog
        fi
fi

##########
# xmessage

if [ "$SEND_XMESSAGE" = "yes" ] ; then
        log "Popping up xmessage window on $XMESSAGE_REMOTE_DISPLAY"
        ( DISPLAY="$XMESSAGE_REMOTE_DISPLAY" \
          xmessage -center "$BRIEFMSG:
$MSG" >/dev/null 2>&1 || \
        error "xmessage failed (remote X display permissions?)" ; ) &
fi

###############
# other command

if [ "$SEND_OTHER_CMD" = "yes" ] ; then
        if [ -z "$RUN_OTHER_CMD" ] ; then
                error "RUN_OTHER_CMD - other command is not defined"
        else
                log "Sending message to forked ($RUN_OTHER_CMD)"
                ( $RUN_OTHER_CMD <"$FMSG" 2>&1 | catlog ) &
        fi
fi

##########
# send SMS
# (it's the last because it can lasts long to send it)

if [ "$SEND_SMS" = "yes" ] ; then
   if [ -z "$SMSSEND_CMD" ] ; then
        error "SMSSEND_CMD not set"
   elif [ -z "$SMSSEND_ARGS" -o -z "$SMSSEND_OK" ] ; then
        error "SMSSEND_ARGS or SMSSEND_OK not set"
   else
     if ! expr "x${SMSSEND_RETRY_INT}x" : "x[1-9][0-9]*x$" >/dev/null ; then
        error "SMSSEND_RETRY_INT not a number, using 30"
        SMSSEND_RETRY_INT=30
     fi
     if ! expr "x${SMSSEND_RETRY}x" : "x[1-9][0-9]*x$" >/dev/null ; then
        error "SMSSEND_RETRY not a number, using 20"
        SMSSEND_RETRY=20
     fi
     if ! expr "x${SMSSEND_CUTCHARS}x" : "x[1-9][0-9]*x$" >/dev/null ; then
        error "SMSSEND_CUTCHARS not a number, using 150"
        SMSSEND_CUTCHARS=150
     fi
     COUNT=0
     log "Sending SMS to $SMSSEND_ARGS"
     SMSMSG=$(echo "$BRIEFMSG: $MSG"|head -c "$SMSSEND_CUTCHARS")
     while "$SMSSEND_CMD" $SMSSEND_ARGS "$SMSMSG" \
             |grep -v "$SMSSEND_OK" >/dev/null ; do
             COUNT=$(($COUNT + 1))
             log "Retrying SMS ... ($COUNT/$SMSSEND_RETRY)"
             if [ "$COUNT" -ge "$SMSSEND_RETRY" ] ; then
                error "Couldn't send SMS (tried $COUNT times)"
                if [ "$SMS_EMAIL_ON_FAIL" != "no" ] ; then
                   if [ -z "$SMS_EMAIL" -a -z "$EMAIL_ADDRESS" ] ; then
                      error "Neither SMS_EMAIL nor EMAIL_ADDRESS defined"
                   else
                      if [ -z "$EMAIL_CMD" ] ; then
                         error "EMAIL_CMD not defined, using 'mail'"
                         EMAIL_CMD=mail
                      fi
                      [ -z "$SMS_EMAIL" ] && SMS_EMAIL="$EMAIL_ADDRESS"
                      log "Sending SMS through email to $SMS_EMAIL"
                      echo "$SMSMSG"|"$EMAIL_CMD" "$SMS_EMAIL" 2>&1|catlog
                   fi
                fi
                break;
             fi
             sleep "$SMSSEND_RETRY_INT"
     done
  fi
fi

rm -f "$FMSG"

log "======================================================"
